@@ -6,18 +6,18 @@ from . import DefaultTable
 
 class table_C_F_F_(DefaultTable.DefaultTable):
 
-	def __init__(self, tag):
+	def __init__(self, tag=None):
 		DefaultTable.DefaultTable.__init__(self, tag)
 		self.cff = cffLib.CFFFontSet()
 		self._gaveGlyphOrder = False
 
 	def decompile(self, data, otFont):
-		self.cff.decompile(BytesIO(data), otFont)
+		self.cff.decompile(BytesIO(data), otFont, isCFF2=False)
 		assert len(self.cff) == 1, "can't deal with multi-font CFF tables."
 
 	def compile(self, otFont):
 		f = BytesIO()
-		self.cff.compile(f, otFont)
+		self.cff.compile(f, otFont, isCFF2=False)
 		return f.getvalue()
 
 	def haveGlyphNames(self):
@@ -44,4 +44,4 @@ class table_C_F_F_(DefaultTable.DefaultTable):
 	def fromXML(self, name, attrs, content, otFont):
 		if not hasattr(self, "cff"):
 			self.cff = cffLib.CFFFontSet()
-		self.cff.fromXML(name, attrs, content)
+		self.cff.fromXML(name, attrs, content, otFont)
