@@ -9,8 +9,26 @@ import sys
 import struct
 import array
 import operator
+import logging
 
 
+log = logging.getLogger(__name__)
+
+
+def _make_map(font, chars, gids):
+	assert len(chars) == len(gids)
+	cmap = {}
+	glyphOrder = font.getGlyphOrder()
+	for char,gid in zip(chars,gids):
+		if gid is 0:
+			continue
+		try:
+			name = glyphOrder[gid]
+		except IndexError:
+			name = font.getGlyphName(gid)
+		cmap[char] = name
+	return cmap
+
 class table__c_m_a_p(DefaultTable.DefaultTable):
 
 	def getcmap(self, platformID, platEncID):
@@ -20,6 +38,36 @@ class table__c_m_a_p(DefaultTable.DefaultTable):
 				return subtable
 		return None # not found
 
+	def getBestCmap(self, cmapPreferences=((3, 10), (0, 6), (0, 4), (3, 1), (0, 3), (0, 2), (0, 1), (0, 0))):
+		"""Return the 'best' unicode cmap dictionary available in the font,
+		or None, if no unicode cmap subtable is available.
+
+		By default it will search for the following (platformID, platEncID)
+		pairs:
+			(3, 10), (0, 6), (0, 4), (3, 1), (0, 3), (0, 2), (0, 1), (0, 0)
+		This can be customized via the cmapPreferences argument.
+		"""
+		for platformID, platEncID in cmapPreferences:
+			cmapSubtable = self.getcmap(platformID, platEncID)
+			if cmapSubtable is not None:
+				return cmapSubtable.cmap
+		return None  # None of the requested cmap subtables were found
+
+	def buildReversed(self):
+		"""Returns a reverse cmap such as {'one':{0x31}, 'A':{0x41,0x391}}.
+
+		The values are sets of Unicode codepoints because
+		some fonts map different codepoints to the same glyph.
+		For example, U+0041 LATIN CAPITAL LETTER A and U+0391
+		GREEK CAPITAL LETTER ALPHA are sometimes the same glyph.
+		"""
+		result = {}
+		for subtable in self.tables:
+			if subtable.isUnicode():
+				for codepoint, name in subtable.cmap.items():
+					result.setdefault(name, set()).add(codepoint)
+		return result
+
 	def decompile(self, data, ttFont):
 		tableVersion, numSubTables = struct.unpack(">HH", data[:4])
 		self.tableVersion = int(tableVersion)
@@ -36,7 +84,10 @@ class table__c_m_a_p(DefaultTable.DefaultTable):
 				format, length = struct.unpack(">HL", data[offset:offset+6])
 
 			if not length:
-				print("Error: cmap subtable is reported as having zero length: platformID %s, platEncID %s,  format %s offset %s. Skipping table." % (platformID, platEncID,format, offset))
+				log.error(
+					"cmap subtable is reported as having zero length: platformID %s, "
+					"platEncID %s, format %s offset %s. Skipping table.",
+					platformID, platEncID, format, offset)
 				continue
 			table = CmapSubtable.newSubtable(format)
 			table.platformID = platformID
@@ -202,26 +253,22 @@ class cmap_format_0(CmapSubtable):
 			assert (data is None and ttFont is None), "Need both data and ttFont arguments"
 		data = self.data # decompileHeader assigns the data after the header to self.data
 		assert 262 == self.length, "Format 0 cmap subtable not 262 bytes"
-		glyphIdArray = array.array("B")
-		glyphIdArray.fromstring(self.data)
-		self.cmap = cmap = {}
-		lenArray = len(glyphIdArray)
-		charCodes = list(range(lenArray))
-		names = map(self.ttFont.getGlyphName, glyphIdArray)
-		list(map(operator.setitem, [cmap]*lenArray, charCodes, names))
+		gids = array.array("B")
+		gids.fromstring(self.data)
+		charCodes = list(range(len(gids)))
+		self.cmap = _make_map(self.ttFont, charCodes, gids)
 
 	def compile(self, ttFont):
 		if self.data:
 			return struct.pack(">HHH", 0, 262, self.language) + self.data
 
-		charCodeList = sorted(self.cmap.items())
-		charCodes = [entry[0] for entry in charCodeList]
-		valueList = [entry[1] for entry in charCodeList]
-		assert charCodes == list(range(256))
-		valueList = map(ttFont.getGlyphID, valueList)
+		cmap = self.cmap
+		assert set(cmap.keys()).issubset(range(256))
+		getGlyphID = ttFont.getGlyphID
+		valueList = [getGlyphID(cmap[i]) if i in cmap else 0 for i in range(256)]
 
-		glyphIdArray = array.array("B", valueList)
-		data = struct.pack(">HHH", 0, 262, self.language) + glyphIdArray.tostring()
+		gids = array.array("B", valueList)
+		data = struct.pack(">HHH", 0, 262, self.language) + gids.tostring()
 		assert len(data) == 262
 		return data
 
@@ -351,7 +398,7 @@ class cmap_format_2(CmapSubtable):
 		# value. In this case the final glyph index = 3+ 42 -> 45 for the final glyphIndex. Whew!
 
 		self.data = b""
-		self.cmap = cmap = {}
+		cmap = {}
 		notdefGI = 0
 		for firstByte in range(256):
 			subHeadindex = subHeaderKeys[firstByte]
@@ -382,17 +429,10 @@ class cmap_format_2(CmapSubtable):
 				# If not subHeader.entryCount, then all char codes with this first byte are
 				# mapped to .notdef. We can skip this subtable, and leave the glyphs un-encoded, which is the
 				# same as mapping it to .notdef.
-		# cmap values are GID's.
-		glyphOrder = self.ttFont.getGlyphOrder()
+
 		gids = list(cmap.values())
 		charCodes = list(cmap.keys())
-		lenCmap = len(gids)
-		try:
-			names = list(map(operator.getitem, [glyphOrder]*lenCmap, gids ))
-		except IndexError:
-			getGlyphName = self.ttFont.getGlyphName
-			names = list(map(getGlyphName, gids ))
-		list(map(operator.setitem, [cmap]*lenCmap, charCodes, names))
+		self.cmap = _make_map(self.ttFont, charCodes, gids)
 
 	def compile(self, ttFont):
 		if self.data:
@@ -700,15 +740,7 @@ class cmap_format_4(CmapSubtable):
 						glyphID = 0  # missing glyph
 					gids.append(glyphID & 0xFFFF)
 
-		self.cmap = cmap = {}
-		lenCmap = len(gids)
-		glyphOrder = self.ttFont.getGlyphOrder()
-		try:
-			names = list(map(operator.getitem, [glyphOrder]*lenCmap, gids ))
-		except IndexError:
-			getGlyphName = self.ttFont.getGlyphName
-			names = list(map(getGlyphName, gids ))
-		list(map(operator.setitem, [cmap]*lenCmap, charCodes, names))
+		self.cmap = _make_map(self.ttFont, charCodes, gids)
 
 	def compile(self, ttFont):
 		if self.data:
@@ -838,23 +870,14 @@ class cmap_format_6(CmapSubtable):
 		firstCode = int(firstCode)
 		data = data[4:]
 		#assert len(data) == 2 * entryCount  # XXX not true in Apple's Helvetica!!!
-		glyphIndexArray = array.array("H")
-		glyphIndexArray.fromstring(data[:2 * int(entryCount)])
+		gids = array.array("H")
+		gids.fromstring(data[:2 * int(entryCount)])
 		if sys.byteorder != "big":
-			glyphIndexArray.byteswap()
+			gids.byteswap()
 		self.data = data = None
 
-		self.cmap = cmap = {}
-
-		lenArray = len(glyphIndexArray)
-		charCodes = list(range(firstCode, firstCode + lenArray))
-		glyphOrder = self.ttFont.getGlyphOrder()
-		try:
-			names = list(map(operator.getitem, [glyphOrder]*lenArray, glyphIndexArray ))
-		except IndexError:
-			getGlyphName = self.ttFont.getGlyphName
-			names = list(map(getGlyphName, glyphIndexArray ))
-		list(map(operator.setitem, [cmap]*lenArray, charCodes, names))
+		charCodes = list(range(firstCode, firstCode + len(gids)))
+		self.cmap = _make_map(self.ttFont, charCodes, gids)
 
 	def compile(self, ttFont):
 		if self.data:
@@ -864,12 +887,14 @@ class cmap_format_6(CmapSubtable):
 		if codes: # yes, there are empty cmap tables.
 			codes = list(range(codes[0], codes[-1] + 1))
 			firstCode = codes[0]
-			valueList = [cmap.get(code, ".notdef") for code in codes]
-			valueList = map(ttFont.getGlyphID, valueList)
-			glyphIndexArray = array.array("H", valueList)
+			valueList = [
+				ttFont.getGlyphID(cmap[code]) if code in cmap else 0
+				for code in codes
+			]
+			gids = array.array("H", valueList)
 			if sys.byteorder != "big":
-				glyphIndexArray.byteswap()
-			data = glyphIndexArray.tostring()
+				gids.byteswap()
+			data = gids.tostring()
 		else:
 			data = b""
 			firstCode = 0
@@ -930,15 +955,7 @@ class cmap_format_12_or_13(CmapSubtable):
 			charCodes.extend(list(range(startCharCode, endCharCode +1)))
 			gids.extend(self._computeGIDs(glyphID, lenGroup))
 		self.data = data = None
-		self.cmap = cmap = {}
-		lenCmap = len(gids)
-		glyphOrder = self.ttFont.getGlyphOrder()
-		try:
-			names = list(map(operator.getitem, [glyphOrder]*lenCmap, gids ))
-		except IndexError:
-			getGlyphName = self.ttFont.getGlyphName
-			names = list(map(getGlyphName, gids ))
-		list(map(operator.setitem, [cmap]*lenCmap, charCodes, names))
+		self.cmap = _make_map(self.ttFont, charCodes, gids)
 
 	def compile(self, ttFont):
 		if self.data:
@@ -1243,9 +1260,8 @@ class cmap_format_14(CmapSubtable):
 		data = bytesjoin(varSelectorRecords) + bytesjoin(data)
 		self.length = 10 + len(data)
 		headerdata = struct.pack(">HLL", self.format, self.length, self.numVarSelectorRecords)
-		self.data = headerdata + data
 
-		return self.data
+		return headerdata + data
 
 
 class cmap_format_unknown(CmapSubtable):
